/* $Id: libtds.c,v 1.7 2006/01/14 13:30:23 kiyohara Exp $ */

#define TDS_VERSION	2

#include <ctype.h>
#include <errno.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <stdint.h>
#include <time.h>
#include <unistd.h>

#include <sys/ioctl.h>

/* XXXX: tricky definitions */
#define UTDS_DO_GARBAGE_CLEAR    

#include <../include/tdsio.h>


static char tds_default_device[] = "/dev/tds0";
static char *typetbl[] = { "9260", "9270", NULL };
static uint32_t libtds_ver = 0x00010000;

struct iotbl {
	int key;
	int value;
};
static struct iotbl Freq[] = {
	{	       (40 * 1000),	TDSIO_SAMPLING_40KHZ	},
	{	      (100 * 1000),	TDSIO_SAMPLING_100KHZ	},
	{	      (200 * 1000),	TDSIO_SAMPLING_200KHZ	},
	{	      (500 * 1000),	TDSIO_SAMPLING_500KHZ	},
	{	 (1 * 1000 * 1000),	TDSIO_SAMPLING_1MHZ	},
	{	 (2 * 1000 * 1000),	TDSIO_SAMPLING_2MHZ	},
	{	 (4 * 1000 * 1000),	TDSIO_SAMPLING_4MHZ	},
	{	 (8 * 1000 * 1000),	TDSIO_SAMPLING_8MHZ	},
	{	(16 * 1000 * 1000),	TDSIO_SAMPLING_16MHZ	},
	{	(32 * 1000 * 1000),	TDSIO_SAMPLING_32MHZ	},
	{	(64 * 1000 * 1000),	TDSIO_SAMPLING_64MHZ	},
	{	0,			-1 			}
};
static struct iotbl Filter[] = {
	{	(16 * 1000 * 1000),	TDSIO_SAMPLING_16M	},
	{	 (8 * 1000 * 1000),	TDSIO_SAMPLING_8M	},
	{	 (4 * 1000 * 1000),	TDSIO_SAMPLING_4M	},
	{	 (2 * 1000 * 1000),	TDSIO_SAMPLING_2M	},
	{	0,			TDSIO_SAMPLING_THRU	}
};
static struct iotbl Bitw[] = {
	{	1,	TDSIO_SAMPLING_1BIT	},
	{	2,	TDSIO_SAMPLING_2BIT	},
	{	4,	TDSIO_SAMPLING_4BIT	},
	{	8,	TDSIO_SAMPLING_8BIT	},
	{	0,	-1	 		}
};
static struct iotbl Chan[] = {
	{	1,	TDSIO_SAMPLING_1CH	},
	{	4,	TDSIO_SAMPLING_4CH	},
	{	0,	-1 			}
};


tdsdev_t *
TdsOpen(char *dev)
{
	char *d;
	tdsdev_t *tds = NULL;

	tds = (tdsdev_t *) malloc(sizeof(tdsdev_t));
	if (tds == NULL)
		goto end;

	if (dev == NULL) {
		d = tds_default_device;
	}
	else {
		d = dev;
	}

	tds->dev = (char *) malloc(sizeof(d));
	if (tds->dev == NULL) {
		free(tds);
		tds = NULL;
		goto end;
	}
	memcpy(tds->dev, d, sizeof(d));

	tds->fd = open(d, O_RDONLY);

	if (tds->fd == -1) {
		free(tds->dev);
		free(tds);
		tds = NULL;
		goto end;
	}

	tds->error = 0;

 end:
	return tds;
}

tdsdev_t *
TdsOpen2(char *dev, char *type)
{
	tdsdev_t *tds;
	int i;
	char *d;

	tds = TdsOpen(dev);
	if (type != NULL) {
		for (i = 0; typetbl[i] != NULL; i++)
			if (strcmp(typetbl[i], type) == 0)
				break;
		tds->type = typetbl[i];
	} else
		tds->type = NULL;

 end:
	return tds;
}


int
TdsClose(tdsdev_t *tds)
{
	if (tds != NULL) {
		close(tds->fd);
		free(tds->dev);
		free(tds);
	}
	return 0;
}


int
TdsInit(tdsdev_t *tds)
{
	int rv;

	if (tds == NULL) {
		rv = EINVAL;
		goto end;
	}

	rv = ioctl(tds->fd, TDSIO_INIT);

	if (rv == -1) {
		rv = errno;
		tds->error = rv;
	}
	else {
		rv = 0;
	}

 end:
	return rv;
}


int
TdsClrBuffer(tdsdev_t *tds)
{
	int rv;

	if (tds == NULL) {
		rv = EINVAL;
		goto end;
	}
		
	rv = ioctl(tds->fd, TDSIO_BUFFER_CLEAR);

	if (rv == -1) {
		rv = errno;
		tds->error = rv;
	}
	else {
		rv = 0;
	}

 end:
	return rv;
}


int
TdsSync1pps(tdsdev_t *tds)
{
	int rv;

	if (tds == NULL) {
		rv = EINVAL;
		goto end;
	}

	rv = ioctl(tds->fd, TDSIO_SYNC_1PPS);

	if (rv == -1) {
		rv = errno;
		tds->error = rv;
	}
	else {
		rv = 0;
	}

 end:
	return rv;
}


int
TdsSyncTime(tdsdev_t *tds)
{
	int rv;

	if (tds == NULL) {
		rv = EINVAL;
		goto end;
	}

	rv = ioctl(tds->fd, TDSIO_SYNC_TIME);

	if (rv == -1) {
		rv = errno;
		tds->error = rv;
	}
	else {
		rv = 0;
	}

 end:
	return rv;
}


#define TDS_MAX_SEC	86400	/* 60 * 60 * 24 */
#define TDS_MAX_DAY 366


int
TdsSetTime(tdsdev_t *tds, tds_time_t *tdstime)
{
	char *d;
	uint32_t data;
	int rv;

	if (tds == NULL) {
		rv = EINVAL;
		goto end;
	}

	if (tdstime == NULL) {		/* set current time */
		struct tm *tm_time;
		time_t curr_time;
		int sec;

		curr_time = time(NULL);
		tm_time = localtime(&curr_time);
		sec = (tm_time->tm_hour * 60 + tm_time->tm_min) * 60
			+ tm_time->tm_sec;
		data = TDS_MAKE_TIME(tm_time->tm_yday + 1, sec);
	}
	else {
		if (tdstime->sec < 0 || tdstime->sec > TDS_MAX_SEC
			|| tdstime->day < 1 || tdstime->day > TDS_MAX_DAY) {
			rv = EINVAL;
			goto end;
		}
		data = TDS_MAKE_TIME(tdstime->day, tdstime->sec);
	}

	rv = ioctl(tds->fd, TDSIO_SET_TIME, &data);

	if (rv == -1) {
		rv = errno;
		tds->error = rv;
	}
	else {
		rv = 0;
	}

 end:
	return rv;
}


int
TdsGetTime(tdsdev_t *tds, tds_time_t *tdstime)
{
	uint32_t data;
	int rv;

	if (tdstime == NULL) {
		rv = EINVAL;
		goto end;
	}

	if (tds == NULL) {
		rv = EINVAL;
		goto end;
	}

	rv = ioctl(tds->fd, TDSIO_GET_TIME, &data);

	if (rv == -1) {
		rv = errno;
		tds->error = rv;
	}
	else {
		tdstime->sec = TDS_GET_SEC(data);
		tdstime->sec_carry = TDS_GET_SEC_CARRY(data);
		tdstime->day = TDS_GET_DAYS(data);
		tdstime->day_carry = TDS_GET_DAYS_CARRY(data);
		rv = 0;
	}

 end:
	return rv;
}


int
TdsSetYear(tdsdev_t *tds, unsigned int *year)
{
	int rv;

	if (tds == NULL) {
		rv = EINVAL;
		goto end;
	}

	if (year == NULL) {
		struct tm *tm_time;
		time_t curr_time;

		curr_time = time(NULL);
		tm_time = localtime(&curr_time);
		tm_time->tm_year &= 0xff;
		rv = ioctl(tds->fd, TDSIO_SET_YEAR, &tm_time->tm_year);
	} else {
		*year &= 0xff;
		rv = ioctl(tds->fd, TDSIO_SET_YEAR, year);
	}

	if (rv == -1) {
		rv = errno;
		tds->error = rv;
	}
	else {
		rv = 0;
	}

 end:
	return rv;
}


int
TdsGetYear(tdsdev_t *tds, unsigned int *year)
{
	int rv;

	if (year == NULL) {
		rv = EINVAL;
		goto end;
	}

	if (tds == NULL) {
		rv = EINVAL;
		goto end;
	}

	rv = ioctl(tds->fd, TDSIO_GET_YEAR, year);

	if (rv == -1) {
		rv = errno;
		tds->error = rv;
	}
	else {
		rv = 0;
	}

 end:
	return rv;
}


int
TdsSetSampling(tdsdev_t *tds, tds_samp_t *samp)
{
	int rv;
	unsigned int flag = 0;

	if (tds == NULL) {
		rv = EINVAL;
		goto end;
	}

	if (samp == NULL) {
		rv = EINVAL;
		goto end;
	}

	if ((samp->freq == 40) && (tolower(samp->unit) == 'k')) {
		flag |= TDS_SAMPLING_40KHZ;
	}
	else if ((samp->freq == 100) && (tolower(samp->unit) == 'k')) {
		flag |= TDS_SAMPLING_100KHZ;
	}
	else if ((samp->freq == 200) && (tolower(samp->unit) == 'k')) {
		flag |= TDS_SAMPLING_200KHZ;
	}
	else if ((samp->freq == 500) && (tolower(samp->unit) == 'k')) {
		flag |= TDS_SAMPLING_500KHZ;
	}
	else if ((samp->freq == 1) && (tolower(samp->unit) == 'm')) {
		flag |= TDS_SAMPLING_1MHZ;
	}
	else if ((samp->freq == 2) && (tolower(samp->unit) == 'm')) {
		flag |= TDS_SAMPLING_2MHZ;
	}
	else if ((samp->freq == 4) && (tolower(samp->unit) == 'm')) {
		flag |= TDS_SAMPLING_4MHZ;
	}
	else if ((samp->freq == 8) && (tolower(samp->unit) == 'm')) {
		flag |= TDS_SAMPLING_8MHZ;
	}
	else if ((samp->freq == 16) && (tolower(samp->unit) == 'm')) {
		flag |= TDS_SAMPLING_16MHZ;
	}
#ifndef	TDS_COMPAT_VLBI111
	else if ((samp->freq == 32) && (tolower(samp->unit) == 'm')) {
		flag |= TDS_SAMPLING_32MHZ;
	}
	else if ((samp->freq == 64) && (tolower(samp->unit) == 'm')) {
		flag |= TDS_SAMPLING_64MHZ;
	}
#endif /* TDS_COMPAT_VLBI111 */
	else {
		rv = EINVAL;
		goto end;
	}
	
	switch (samp->width) {
	case 1:
		flag |= TDS_SAMPLING_1BIT;
		break;
	case 2:
		flag |= TDS_SAMPLING_2BIT;
		break;
	case 4:
		flag |= TDS_SAMPLING_4BIT;
		break;
	case 8:
		flag |= TDS_SAMPLING_8BIT;
		break;
	default:
		rv = EINVAL;
		goto end;
	}

	switch (samp->channel) {
	case 1:
		flag |= TDS_SAMPLING_1CH;
#ifndef	TDS_COMPAT_VLBI111
		switch (samp->freq) {
		case 1:
		case 2:
		case 4:
		case 8:
		case 16:
		case 32:
		case 64:
			if ((samp->freq * samp->width) > 128) {
				rv = EINVAL;
				goto end;
			}
			break;
		default:
			break;
		}
#endif /* TDS_COMPAT_VLBI111 */
		break;
	case 4:
		flag |= TDS_SAMPLING_4CH;
#ifndef	TDS_COMPAT_VLBI111
		switch (samp->freq) {
		case 1:
		case 2:
		case 4:
		case 8:
		case 16:
		case 32:
		case 64:
			if ((samp->freq * samp->width) > 32) {
				rv = EINVAL;
				goto end;
			}
			break;
		default:
			break;
		}
#else /* TDS_COMPAT_VLBI111 */
		if (samp->freq < 40) {
			if ((samp->freq * samp->width) > 32) {
				rv = EINVAL;
				goto end;
			}
		}
#endif /* TDS_COMPAT_VLBI111 */
		break;
	default:
		rv = EINVAL;
		goto end;
	}
		
	rv = ioctl(tds->fd, TDSIO_SET_SAMPLING, &flag);

	if (rv == -1) {
		rv = errno;
		tds->error = rv;
	}
	else {
		rv = 0;
	}

 end:
	return rv;
}


int
TdsGetSampling(tdsdev_t *tds, tds_samp_t *samp)
{
	int rv;
	unsigned int flag;

	if (tds == NULL) {
		rv = EINVAL;
		goto end;
	}

	if (samp == NULL) {
		rv = EINVAL;
		goto end;
	}

	rv = ioctl(tds->fd, TDSIO_GET_SAMPLING, &flag);

	if (rv == -1) {
		rv = errno;
		tds->error = rv;
	}
	else {
		unsigned char f = TDS_GET_SAMP_FREQ(flag);
		unsigned char w = TDS_GET_SAMP_WIDTH(flag);
		unsigned char c = TDS_GET_SAMP_CH(flag);

		rv = 0;

		switch (f) {
		case TDS_SAMPLING_40KHZ:
			samp->freq = 40;
			samp->unit =  'k';
			break;
		case TDS_SAMPLING_100KHZ:
			samp->freq = 100;
			samp->unit = 'k';
			break;
		case TDS_SAMPLING_200KHZ:
			samp->freq = 200;
			samp->unit = 'k';
			break;
		case TDS_SAMPLING_500KHZ:
			samp->freq = 500;
			samp->unit = 'k';
			break;
		case TDS_SAMPLING_1MHZ:
			samp->freq = 1;
			samp->unit = 'M';
			break;
		case TDS_SAMPLING_2MHZ:
			samp->freq = 2;
			samp->unit = 'M';
			break;
		case TDS_SAMPLING_4MHZ:
			samp->freq = 4;
			samp->unit = 'M';
			break;
		case TDS_SAMPLING_8MHZ:
			samp->freq = 8;
			samp->unit = 'M';
			break;
		case TDS_SAMPLING_16MHZ:
			samp->freq = 16;
			samp->unit = 'M';
			break;
#ifndef	TDS_COMPAT_VLBI111
		case TDS_SAMPLING_32MHZ:
			samp->freq = 32;
			samp->unit = 'M';
			break;
		case TDS_SAMPLING_64MHZ:
			samp->freq = 64;
			samp->unit = 'M';
			break;
#endif	/* TDS_COMPAT_VLBI111 */
		default:
			rv = -1;
			break;
		}

		switch (w) {
		case TDS_SAMPLING_1BIT:
			samp->width = 1;
			break;
		case TDS_SAMPLING_2BIT:
			samp->width = 2;
			break;
		case TDS_SAMPLING_4BIT:
			samp->width = 4;
			break;
		case TDS_SAMPLING_8BIT:
			samp->width = 8;
			break;
		default:
			rv = -1;
			break;
		}

		switch (c) {
		case TDS_SAMPLING_1CH:
			samp->channel = 1;
			break;
		case TDS_SAMPLING_4CH:
			samp->channel = 4;
			break;
		default:
			rv = -1;
			break;
		}

	}

 end:
	return rv;
}

int
TdsSetSampling2(tdsdev_t *tds,
    int *fsample, int *filter, int *resolutionbit, int *chmode)
{
	uint32_t value;
	int rv = 0, i;

	if (tds == NULL) {
		rv = EINVAL;
		goto end;
	}

	if (fsample != NULL) {
		value = *fsample;
		if ((rv = ioctl(tds->fd, TDSIO_SET_FSAMPLE, &value)) == -1) {
			rv = errno;
			tds->error = rv;
		}
	}

	if (filter != NULL) {
		if ((rv = ioctl(tds->fd, TDSIO_GET_FILTER, &value)) == 0) {
			/* assume we can TDSIO_SET_FILTER */
			value = *filter;
			if ((rv = ioctl(tds->fd, TDSIO_SET_FILTER, &value)) ==
			    -1) {
				rv = errno;
				tds->error = rv;
			}
		}
	}

	if (resolutionbit != NULL) {
		value = *resolutionbit;
		if ((rv = ioctl(tds->fd, TDSIO_SET_RESOLUTIONBIT, &value)) ==
		    -1) {
			rv = errno;
			tds->error = rv;
		}
	}

	if (chmode != NULL) {
		value = *chmode;
		if ((rv = ioctl(tds->fd, TDSIO_SET_CHMODE, &value)) == -1) {
			rv = errno;
			tds->error = rv;
		}
	}

 end:
	return rv;
}


int
TdsGetSampling2(tdsdev_t *tds,
    int *fsample, int *filter, int *resolutionbit, int *chmode)
{
	uint32_t value;
	int rv = 0, i;

	if (tds == NULL) {
		rv = EINVAL;
		goto end;
	}

	if (fsample != NULL) {
		if ((rv = ioctl(tds->fd, TDSIO_GET_FSAMPLE, &value)) == -1) {
			rv = errno;
			tds->error = rv;
		}
		*fsample = value;
	}

	if (filter != NULL) {
		if ((rv = ioctl(tds->fd, TDSIO_GET_FILTER, &value)) == 0) {
			*filter = value;
		} else
			*filter = -1;
	}

	if (resolutionbit != NULL) {
		if ((rv = ioctl(tds->fd, TDSIO_GET_RESOLUTIONBIT, &value)) ==
		    -1) {
			rv = errno;
			tds->error = rv;
		}
		*resolutionbit = value;
	}

	if (chmode != NULL) {
		if ((rv = ioctl(tds->fd, TDSIO_GET_CHMODE, &value)) == -1) {
			rv = errno;
			tds->error = rv;
		}
		*chmode = value;
	}

 end:
	return rv;
}


int
TdsSetDcoffset(tdsdev_t *tds, int *num, unsigned int *dcoff)
{
	struct tdsio tdsio;
	int i, rv = 0;

	for (i = 0; i < *num; i++) {
		tdsio.key = i;
		tdsio.value = *dcoff;
		if ((rv = ioctl(tds->fd, TDSIO_SET_DCOFFSET, &tdsio)) == -1) {
			rv = errno;
			tds->error = rv;
			goto end;
		}
		dcoff ++;
	}
 end:
	*num = i;	/* overwrite successed num */
	return rv;
}


int
TdsGetDcoffset(tdsdev_t *tds, int *num, unsigned int *dcoff)
{
	struct tdsio tdsio;
	int i, rv = 0;

	for (i = 0; i < *num; i++) {
		tdsio.key = i;
		if ((rv = ioctl(tds->fd, TDSIO_GET_DCOFFSET, &tdsio)) == -1) {
			rv = errno;
			tds->error = rv;
			goto end;
		}
		*dcoff = tdsio.value;
		dcoff ++;
	}
 end:
	*num = i;	/* overwrite successed num */
	return rv;
}


int
TdsStatus(tdsdev_t *tds, unsigned int *status)
{
	int rv;

	if (tds == NULL) {
		rv = EINVAL;
		goto end;
	}

	if (status == NULL) {
		rv = EINVAL;
		goto end;
	}

	rv = ioctl(tds->fd, TDSIO_GET_STATUS, status);

	if (rv == -1) {
		rv = errno;
		tds->error = rv;
	}
	else {
		rv = 0;
	}

 end:
	return rv;
}


int
TdsStart(tdsdev_t *tds)
{
	int rv;

	if (tds == NULL) {
		rv = EINVAL;
		goto end;
	}

	rv = ioctl(tds->fd, TDSIO_SAMPLING_START);

	if (rv == -1) {
		rv = errno;
		tds->error = rv;
	}
	else {
		rv = 0;
	}

 end:
	return rv;
}


int
TdsStop(tdsdev_t *tds)
{
	int rv;

	if (tds == NULL) {
		rv = EINVAL;
		goto end;
	}

	rv = ioctl(tds->fd, TDSIO_SAMPLING_STOP);

	if (rv == -1) {
		rv = errno;
		tds->error = rv;
	}
	else {
		rv = 0;
	}

 end:
	return rv;
}


int
TdsGetData(tdsdev_t *tds, unsigned int *data)
{
	int rv;

	if (tds == NULL) {
		rv = EINVAL;
		goto end;
	}

	if (data == NULL) {
		rv = EINVAL;
		goto end;
	}

	rv = read(tds->fd, data, sizeof(*data));

	if (rv == -1) {
		rv = errno;
		tds->error = rv;
	}
	else {
		rv = 0;
	}

 end:
	return rv;
}


int
TdsRead(tdsdev_t *tds, int *num, unsigned char *data)
{
	int rv;

	if (tds == NULL) {
		rv = EINVAL;
		goto end;
	}

	if ((num < 0) || (data == NULL)) {
		rv = EINVAL;
		goto end;
	}

	rv = read(tds->fd, data, *num);

	if (rv == -1) {
		rv = errno;
		tds->error = rv;
	}
	else {
		*num = rv;
		rv = 0;
	}

 end:
	return rv;
}


int
TdsSetAux(tdsdev_t *tds, int *auxsize, char *auxbuf)
{
	struct tdsauxio auxio;
	int rv = 0;

	if (tds == NULL) {
		rv = EINVAL;
		goto end;
	}
	if (auxsize == NULL ||
	    !(*auxsize > 0 && *auxsize <= sizeof (auxio.auxfield))) {
		rv = EINVAL;
		goto end;
	}
	if (auxbuf == NULL) {
		rv = EINVAL;
		goto end;
	}

	auxio.auxsize = *auxsize;
	memcpy(&auxio.auxfield, auxbuf, *auxsize);
	if ((rv = ioctl(tds->fd, TDSIO_SET_AUX, &auxio)) == -1) {
		rv = errno;
		tds->error = rv;
	}

 end:
	return rv;
}

int
TdsSetAuxFile(tdsdev_t *tds, char *auxfile)
{
	int auxsize, fd, rv = 0;
	char auxbuf[256];

	if ((fd = open(auxfile, O_RDONLY)) == -1) {
		rv = ENOENT;
		goto end;
	}
	if ((auxsize = read(fd, auxbuf, sizeof (auxbuf))) == -1) {
		rv = EIO;
		goto end;
	}
	close(fd);
	TdsSetAux(tds, &auxsize, auxbuf);

 end:
	return rv;
}


int
TdsGetAux(tdsdev_t *tds, int *auxsize, char *auxbuf)
{
	struct tdsauxio auxio;
	int rv = 0;

	if (tds == NULL) {
		rv = EINVAL;
		goto end;
	}
	if (auxsize == NULL) {
		rv = EINVAL;
		goto end;
	}
	if (auxbuf == NULL) {
		rv = EINVAL;
		goto end;
	}

	if ((rv = ioctl(tds->fd, TDSIO_GET_AUX, &auxio)) == -1) {
		rv = errno;
		tds->error = rv;
	}
	*auxsize = auxio.auxsize;
	memcpy(auxbuf, &auxio.auxfield, auxio.auxsize);

 end:
	return rv;
}

int
TdsGetAuxFile(tdsdev_t *tds, char *auxfile)
{
	int auxsize, fd, rv = 0;
	char auxbuf[256];

	TdsGetAux(tds, &auxsize, auxbuf);
	if ((fd = open(auxfile, O_RDONLY)) == -1) {
		rv = ENOENT;
		goto end;
	}
	while (auxsize > 0) {
		if ((rv = write(fd, auxbuf, auxsize)) == -1) {
			rv = EIO;
			goto end;
		}
		auxsize -= rv;
	}
	close(fd);

 end:
	return rv;
}


#ifdef UTDS_DO_GARBAGE_CLEAR
int
TdsGetRefound(tdsdev_t *tds, unsigned int *refound)
{
	int rv;

	if (tds == NULL) {
		rv = EINVAL;
		goto end;
	}

	if (refound == NULL) {
		rv = EINVAL;
		goto end;
	}

	rv = ioctl(tds->fd, TDSIO_GET_REFOUND, refound);

	if (rv == -1) {
		rv = errno;
		tds->error = rv;
	}
	else {
		rv = 0;
	}

 end:
	return rv;
}
#endif


char *tds_error_messages[] = {
	/*  0 */	"no error",
	/*  1 */	"unknown error",
	/*  2 */	"requested operation or supplied parameter is not "
			    "supported",
	/*  3 */	"sampling not started yet",
	/*  4 */	"sampling already started",
	/*  5 */	"sampling flag(freqency/width/channel) is not set yet",
	/*  6 */	"device FIFO overflow",
	/*  7 */	"1pps input signal not detected",
	/*  8 */	"10MHz input signal not detected",
	/*  9 */	"time input signal not detected",
	/* 10 */	"not synchronized to 1pps yet",
	/* 11 */	"not synchronized to (or set) time yet",
	/* 12 */	"read operation is now in progress",
	/* 13 */	"Ext card not exist",
	/* 14 */	"no DMA buffer found",
	/* 15 */	"data sampler I/O error",
	/* 16 */	"DMA error",
	/* 17 */	"Operation timed out",
	/* 18 */	"alignement error",
	/* 19 */	"header no found",
	/* 20 */	"header re-found",
};


int
TdsError(tdsdev_t *tds, int *err, char **msg)
{
	int rv;

	if (tds == NULL) {
		rv = EINVAL;
		goto end;
	}

	if (err == NULL) {
		rv = EINVAL;
		goto end;
	}

	rv = ioctl(tds->fd, TDSIO_GET_ERROR, err);

	if (rv == -1) {
		rv = errno;
		tds->error = rv;
	}
	else {
		rv = 0;
	}

	*msg = tds_error_messages[*err];

 end:
	return rv;
}


int
TdsVersion(tdsdev_t *tds, char **msg)
{
	*msg = (char *)_szVer;
	return 0;
}


int
TdsGetVersion(tdsdev_t *tds, int *drv, int *lib)
{
	int rv;

	if (tds == NULL) {
		rv = EINVAL;
		goto end;
	}

	if (drv == NULL || lib == NULL) {
		rv = EINVAL;
		goto end;
	}

	rv = ioctl(tds->fd, TDSIO_GET_VERSION, drv);

	if (rv == -1) {
		rv = errno;
		tds->error = rv;
	}
	else {
		rv = 0;
	}
	*lib = libtds_ver;

end:
	return rv;
}

